from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

API_KEY = getattr(settings, 'AMPLITUDE_API_KEY')
if not API_KEY:
    raise ImproperlyConfigured('"AMPLITUDE_API_KEY" is not set')

INCLUDE_USER_DATA = getattr(settings, 'AMPLITUDE_INCLUDE_USER_DATA', False)  # NOQA: E501
INCLUDE_GROUP_DATA = getattr(settings, 'AMPLITUDE_INCLUDE_GROUP_DATA', False)  # NOQA: E501
IGNORE_URLS = getattr(settings, 'AMPLITUDE_IGNORE_URLS', [])
if not isinstance(IGNORE_URLS, list):
    error = '"AMPLITUDE_IGNORE_URLS" must be a list of URLs or URL names'
    raise ImproperlyConfigured(error)

MIN_ID_LENGTH = getattr(settings, 'AMPLITUDE_MIN_ID_LENGTH', None)
if MIN_ID_LENGTH and not isinstance(MIN_ID_LENGTH, int):
    raise ImproperlyConfigured('"AMPLITUDE_MIN_ID_LENGTH" must be an integer')

installed_apps = getattr(settings, 'INSTALLED_APPS')
middleware = getattr(settings, 'MIDDLEWARE')
missing_session_settings = (
    'django.contrib.sessions' not in installed_apps
    or 'django.contrib.sessions.middleware.SessionMiddleware' not in middleware  # NOQA: E503, E501
)
if missing_session_settings:
    error = ('django.contrib.sessions must be included in INSTALLED_APPS and '
             'django.contrib.sessions.middleware.SessionMiddleware in '
             'MIDDLEWARE to be possible to track users')
    raise ImproperlyConfigured(error)

missing_auth_settings = (
    'django.contrib.auth' not in installed_apps
    or 'django.contrib.auth.middleware.AuthenticationMiddleware' not in middleware  # NOQA: E503, E501
)
if INCLUDE_USER_DATA and missing_auth_settings:
    error = ('django.contrib.auth must be in INSTALLED_APPS and '
             'django.contrib.auth.middleware.AuthenticationMiddleware in '
             'MIDDLEWARE when "AMPLITUDE_INCLUDE_USER_DATA" is turned on')
    raise ImproperlyConfigured(error)
if INCLUDE_GROUP_DATA and missing_auth_settings:
    error = ('django.contrib.auth must be in INSTALLED_APPS and '
             'django.contrib.auth.middleware.AuthenticationMiddleware in '
             'MIDDLEWARE when "AMPLITUDE_INCLUDE_GROUP_DATA" is turned on')
    raise ImproperlyConfigured(error)



from django import get_version
from packaging import version

from .amplitude import Amplitude  # NOQA: F401

"""
To stop waring:
    RemovedInDjango41Warning: 'amplitude' defines
    default_app_config = 'amplitude.apps.AmplitudeConfig'. Django now detects
    this configuration automatically. You can remove default_app_config.
"""
if version.parse(get_version()) < version.parse('3.2'):  # pragma: no cover
    default_app_config = "amplitude.apps.AmplitudeConfig"



import logging
import time
from typing import Any, Dict, List

import httpx
from django.contrib.auth import get_user_model
from django.http import HttpRequest
from django.urls import resolve

from . import settings as app_settings
from .utils import get_client_ip, get_user_agent

try:
    from django.contrib.gis.geoip2 import GeoIP2  # type: ignore
except ImportError:
    CAN_GEOIP = False
else:  # pragma: no cover
    CAN_GEOIP = True

log = logging.getLogger(__name__)


class AmplitudeException(Exception):
    pass


class Amplitude():

    def __init__(
        self,
        api_key: str = None,
        include_user_data: bool = None,
        include_group_data: bool = None,
        min_id_length: int = None,
    ):
        if not api_key:
            api_key = app_settings.API_KEY
        if include_user_data is None:
            include_user_data = app_settings.INCLUDE_USER_DATA
        if include_group_data is None:
            include_group_data = app_settings.INCLUDE_GROUP_DATA
        if not min_id_length:
            min_id_length = app_settings.MIN_ID_LENGTH

        self.url = 'https://api.amplitude.com/2/httpapi'
        self.api_key = api_key
        self.include_user_data = include_user_data
        self.include_group_data = include_group_data
        self.min_id_length = min_id_length

    def send_events(self, events: List[Dict[str, Any]]) -> dict:
        """
        https://developers.amplitude.com/docs/http-api-v2
        """
        events = [self.clean_event(event) for event in events]
        kwargs: Dict[str, Any] = {
            'url': self.url,
            'method': 'POST',
            'json': {
                'events': events,
                'api_key': self.api_key
            }
        }
        if self.min_id_length is not None:
            kwargs['json']['options'] = {
                'min_id_length': self.min_id_length
            }
        response = httpx.request(**kwargs)
        try:
            response.raise_for_status()
        except httpx.HTTPError as e:
            raise AmplitudeException(e)

        return response.json()

    def clean_event(self, event: dict) -> dict:
        for key, value in event.items():
            if isinstance(value, dict):
                event[key] = {k: v for k, v in value.items() if v not in [None, [], '', {}]}  # NOQA: E501

        event = {k: v for k, v in event.items() if v not in [None, [], '', {}]}

        return event

    def build_event_data(
        self, event_type: str,
        request: HttpRequest,
        event_properties: dict = {},
        **kwargs
    ) -> dict:
        """
        Build event data using a Django request object
        """
        event: Dict[str, Any] = {
            'device_id': request.session.get('amplitude_device_id'),
            'session_id': request.session.get('amplitude_session_id'),
            'event_type': event_type,
            'time': int(round(time.time() * 1000)),
            'ip': get_client_ip(request),
            'language': getattr(request, 'LANGUAGE_CODE', ''),
            'app_version': kwargs.get('app_version'),
            'carrier': kwargs.get('carrier'),
            'dma': kwargs.get('dma'),
            'price': kwargs.get('price'),
            'quantity': kwargs.get('quantity'),
            'revenue': kwargs.get('revenue'),
            'productId': kwargs.get('productId'),
            'revenueType': kwargs.get('revenueType'),
            'idfa': kwargs.get('idfa'),
            'idfv': kwargs.get('idfv'),
            'adid': kwargs.get('adid'),
            'android_id': kwargs.get('android_id'),
            'event_id': kwargs.get('event_id'),
            'insert_id': kwargs.get('insert_id'),
        }

        if event_properties:
            event['event_properties'] = event_properties
        else:
            event['event_properties'] = self.event_properties_from_request(request)  # NOQA: E501

        try:
            event['user_id'] = f'{request.user.pk:05}'
        except (AttributeError, TypeError):
            pass
        event['user_properties'] = self.user_properties_from_request(request)
        event['groups'] = self.group_from_request(request)

        device_data = self.device_data_from_request(request)
        event.update(device_data)

        location_data = self.location_data_from_ip_address(event['ip'])
        event.update(location_data)

        return event

    def event_properties_from_request(self, request: HttpRequest) -> dict:
        url_name = resolve(request.path_info).url_name
        event_properties = {
            'url': request.path,
            'url_name': url_name,
            'method': request.method,
            'params': dict(request.GET),
            'scheme': request.scheme,
            'content_type': request.content_type,
            'content_params': request.content_params,
            'content_length': request.META.get('CONTENT_LENGTH'),
            'http_accept': request.META.get('HTTP_ACCEPT'),
            'http_accept_encoding': request.META.get('HTTP_ACCEPT_ENCODING'),
            'http_accept_language': request.META.get('HTTP_ACCEPT_LANGUAGE'),
            'http_host': request.META.get('HTTP_HOST'),
            'referer': request.META.get('HTTP_REFERER'),
            'server_name': request.META.get('SERVER_NAME'),
            'server_port': request.META.get('SERVER_PORT'),
        }
        if request.resolver_match:
            event_properties['kwargs'] = request.resolver_match.kwargs
        return event_properties

    def user_properties_from_request(self, request: HttpRequest) -> dict:
        try:
            request.user.is_authenticated
        except AttributeError:
            return {}

        if not self.include_user_data or not request.user.is_authenticated:
            return {}

        User = get_user_model()
        user = User.objects.get(pk=request.user.pk)

        user_data = {
            'username': user.get_username(),
            'email': user.email,
            'full_name': user.get_full_name(),
            'is_staff': user.is_staff,
            'is_superuser': user.is_superuser,
        }
        if user.last_login:
            user_data['last_login'] = user.last_login.isoformat()
        if user.date_joined:
            user_data['date_joined'] = user.date_joined.isoformat()
        return user_data

    def group_from_request(self, request: HttpRequest) -> list:
        try:
            request.user.is_authenticated
        except AttributeError:
            return []

        if not self.include_group_data or not request.user.is_authenticated:
            return []

        User = get_user_model()
        user = User.objects.get(pk=request.user.pk)
        groups = user.groups.all().values_list('name', flat=True)
        return list(groups)

    def location_data_from_ip_address(self, ip_address: str) -> dict:
        location_data: dict = {}

        if not ip_address or not CAN_GEOIP:
            return location_data

        # pip install geoip2
        # https://pypi.org/project/geoip2/
        # from django.contrib.gis.geoip2 import GeoIP2
        g = GeoIP2()
        location = g.city(ip_address)
        location_data['country'] = location['country_name']
        location_data['city'] = location['city']
        lat_lon = g.lat_lon(ip_address)
        location_data['location_lat'] = lat_lon[0]
        location_data['location_lng'] = lat_lon[1]
        return location_data

    def device_data_from_request(self, request: HttpRequest) -> dict:
        device_data: dict = {}

        user_agent = get_user_agent(request)
        if not user_agent:
            return device_data

        device_data['os_name'] = user_agent.os.family
        device_data['os_version'] = user_agent.os.version_string
        device_data['platform'] = user_agent.device.family
        # device_data['device_brand']
        device_data['device_manufacturer'] = user_agent.device.brand
        device_data['device_model'] = user_agent.device.model
        return device_data



import logging
from time import time
from uuid import uuid4

from django.urls import resolve

from . import Amplitude, settings
from .amplitude import AmplitudeException

log = logging.getLogger(__name__)
amplitude = Amplitude()


class SessionInfo(object):
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if not request.session.get('amplitude_device_id'):
            request.session['amplitude_device_id'] = str(uuid4())
        if not request.session.get('amplitude_session_id'):
            request.session['amplitude_session_id'] = int(time()) * 1000
        return self.get_response(request)


class SendPageViewEvent(object):
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        # Ignore URLs can be either a url path or url name
        if request.path_info in settings.IGNORE_URLS:
            return self.get_response(request)
        current_url = resolve(request.path_info)
        if current_url.url_name in settings.IGNORE_URLS:
            return self.get_response(request)

        event = amplitude.build_event_data(
            event_type='Page view', request=request
        )
        try:
            amplitude.send_events(events=[event])
        except AmplitudeException as e:
            log.error(f'Unable to send page view event due to - {e}')
        return self.get_response(request)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['amplitude']

package_data = \
{'': ['*']}

install_requires = \
['Django>=2.1', 'httpx>=0.13.2,<0.14.0', 'user-agents>=2.1,<3.0']

setup_kwargs = {
    'name': 'django-amplitude',
    'version': '0.7.1',
    'description': 'Integration between Django and Amplitude',
    'long_description': "# Django Amplitude\n\nIntegration between Django and [Amplitude.com](https://amplitude.com/) to help send events via the [Amplitude HTTP API (v2)](https://developers.amplitude.com/docs/http-api-v2)\n\n\n## Quick start\n\n### Installation\n\n```bash\npip install django-amplitude\n```\n\nAdd `amplitude` to your `INSTALLED_APPS`. If they are not already the Django `sessions` app must also be added:\n\n```python\nINSTALLED_APPS = [\n    ...\n    'django.contrib.sessions',\n    ...\n    'amplitude',\n]\n```\n\nIf you do not have it already you must also add the Django `django.contrib.sessions.middleware.SessionMiddleware`. Then add the ampliturde `SessionInfo` middleware after the `SessionMiddleware`:\n```python\nMIDDLEWARE = [\n    ...\n    'django.contrib.sessions.middleware.SessionMiddleware',\n    ...\n    'amplitude.middleware.SessionInfo',\n]\n```\n\nNow set your Amplitude API key and user / group options in your `settings.py`:\n```python\n# Settings > Projects > <Your project> > General > API Key\nAMPLITUDE_API_KEY = '<amplitude-project-api-key>'\n\n# You can also choose if you want to include user and group data (Default False)\nAMPLITUDE_INCLUDE_USER_DATA = False\nAMPLITUDE_INCLUDE_GROUP_DATA = False\n```\n\n*Note: If you want to include user or group data you must ensure the [Django auth is setup correctly](https://docs.djangoproject.com/en/3.0/topics/auth/#installation). This includes adding `django.contrib.auth` and `django.contrib.contenttypes` to `INSTALLED_APPS` and `django.contrib.auth.middleware.AuthenticationMiddleware` to `MIDDLEWARE`*.\n\n\nFor more information on the above settings see the [configuration settings](#configuration-settings) section.\n\n\n\n## Usage\n\n### Page view events\n\nIf you want to send an event to Amplitude on every page view you can use the django-amplitude `SendPageViewEvent` middleware to your `MIDDLEWARE` in your Django settings.\n\nThis will automatically create an event called `Page view` with all the information it's possible to pull from the Django request object such as URL path and parameters, user agent info, IP info, user info etc.\n\nIt must be placed after the `amplitude.middleware.SessionInfo` middleware:\n\n```python\nMIDDLEWARE = [\n    'django.contrib.sessions.middleware.SessionMiddleware',\n    ...\n    'amplitude.middleware.SessionInfo',\n    'amplitude.middleware.SendPageViewEvent',\n]\n```\n\nWhile using `SendPageViewEvent` if there are certain URLs that you do not want to update Amplitude with you can use the `AMPLITUDE_IGNORE_URLS` setting. This setting take a list of relative urls or URL names. Relative URLS most match exactly so need to start with a forward slash (`/`)\n\n```python\nAMPLITUDE_IGNORE_URLS = ['my_url_name', '/testurl']\n```\n\n\n### Sending events manually\n\nIf you want to send your own events:\n```python\nfrom amplitude import Amplitude\n\namplitude = Amplitude()\nevent_data = amplitude.build_event_data(\n    event_type='Some event type',\n    request=request,\n)\namplitude.send_events([event_data])\n```\n\nThe above request will include URL and HTTP header info in the `event_properties`. If you want to override the event properties you can pass them through to `build_event_data`:\n\n```python\nevent_data = amplitude.build_event_data(\n    event_type='User purchase',\n    request=request,\n    event_properties={'products': ['laptop', 'phone']}\n)\n```\n\n### build_event_data missing event data keys\n\nThe `build_event_data` method (and in extension the `SendPageViewEvent` middleware) currently does not send the following keys from `UploadRequestBody` type in [Amplitude HTTP API (v2)](https://developers.amplitude.com/docs/http-api-v2):\n\n* event_id\n* app_version\n* carrier\n* price\n* quantity\n* revenue\n* productId\n* revenueType\n* idfa\n* idfv\n* adid\n* android_id\n* dma\n* insert_id\n\nIf you want to record an event in Amplitude with any of these keys you must use build and send your own event data using `amplitude.build_event_data` where you can pass any of the above as kwargs:\n\n```python\namplitude = Amplitude()\nevent_data = amplitude.build_event_data(\n    event_type='Some event type',\n    request=request,\n    app_version='1.0.0',\n)\namplitude.send_events([event_data])\n```\n\n\n### Building you own event\n\nIf you are not happy with the data from `build_event_data` you can build you own event data based on the `UploadRequestBody` type in [Amplitude HTTP API (v2)](https://developers.amplitude.com/docs/http-api-v2). If you want to do this There are a few helper functions to build different parts of the event data from the Django request object:\n\n```python\namplitude.event_properties_from_request(request)  # Gets URL and HTTP header data\namplitude.device_data_from_request(request)  # Gets device info from user agent\namplitude.user_properties_from_request(request)  # Gets info from user model\namplitude.group_from_request(request)  # Gets the list of groups a user is in\n\namplitude.location_data_from_ip_address(ip_address)  # Gets location data from IP if GeoIP2 is setup\n```\n\n* `user_properties_from_request` will return an empty dict if `AMPLITUDE_INCLUDE_USER_DATA` is `False`\n* `group_from_request` will return an empty dict if `AMPLITUDE_INCLUDE_GROUP_DATA` is `False`\n\n\n#### Configuration settings\n\nBelow are the different settings that can be overridden. To do so place the setting into your `settings.py`.\n\n```python\n# This variable is required when amplitude is added to INSTALLED_APPS\nAMPLITUDE_API_KEY = '<amplitude-project-api-key>'\n\n# If the users Django user information is included in the Amplitude event.\n# This includes - username, email, full_name, is_staff, is_superuser\nAMPLITUDE_INCLUDE_USER_DATA = False\n\n# If the groups the user is a member of is included in the Amplitude event.\n# A list of the group names will be sent in the request.\nAMPLITUDE_INCLUDE_GROUP_DATA = False\n\n# A list of URLs which `SendPageViewEvent` middleware should not run for.\n# Each item in the list can be either a URL or url name\nAMPLITUDE_IGNORE_URLS = ['home', '/please/ignore/']\n\n# The minimum permitted length for user_id & device_id fields\n# https://developers.amplitude.com/docs/http-api-v2#properties-2\nAMPLITUDE_MIN_ID_LENGTH = None\n```\n",
    'author': 'Matt Pye',
    'author_email': 'pyematt@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pyepye/django-amplitude',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
